#ifndef RGMAAPPENDER_H
#define RGMAAPPENDER_H
#include "rgma/RGMADateFormat.h"
#include "rgma/RGMATimeFormat.h"
#include "rgma/Constants.h"
#include "log4cxx/spi/filter.h"
#include "log4cxx/spi/loggingevent.h"
#include "log4cxx/appenderskeleton.h"
#include "log4cxx/helpers/dateformat.h"
#include "rgma/PrimaryProducer.h"

using namespace log4cxx;
using namespace log4cxx::helpers;
using namespace log4cxx::spi;

using glite::rgma::PrimaryProducer;
namespace rgma
{
    namespace log4cpp
    {
        class RGMAAppender;
        typedef helpers::ObjectPtrT<RGMAAppender> RGMAAppenderPtr;
        /**
         * Log4cxx Appender for inserting values to R-GMA 
         * 
         */
        class RGMAAppender : public AppenderSkeleton
        {

            DECLARE_LOG4CXX_OBJECT(RGMAAppender)

                BEGIN_LOG4CXX_CAST_MAP()
                LOG4CXX_CAST_ENTRY(RGMAAppender)
                LOG4CXX_CAST_ENTRY_CHAIN(AppenderSkeleton)
                END_LOG4CXX_CAST_MAP()
                public:
                RGMAAppender::RGMAAppender();
                /**
                 * Constructor for Appender.
                 * @param name The name of this Appender.
                 **/
                RGMAAppender(const std::string& name);

                virtual ~RGMAAppender();
                /**
                 * append logging event
                 * @param event  The LoggingEvent to log.
                 **/

                virtual void append(const LoggingEventPtr& event);
                /**
                 * Method called by the log4cxx system once all the properties have
                 * been initialised from the config file. The Producer is created
                 * here because the predicate depends upon the JobName thats
                 * parsed from the log4j properties file.
                 */
                void activateOptions();

                /**
                 * overridden method which sets the JobName and Filter options
                 *
                 */
                void setOption(const String& option, const String& value);

                /**
                 * Indicates this appender should format any logging events. In this
                 * case the dafault log4j formatter will be used.
                 */
                bool requiresLayout()const;
                /**
                 * Called when the log4j system terminates - provides hook
                 * in-order to cleanup the rgma producer.
                 */
                void close();

            private:
                PrimaryProducer* m_producer;
                /**
                 * Attempts to create a new R-GMA producer that will publish the
                 * log4cxx events.
                 */
                void createProducer();
                std::string m_jobName;
                std::string m_filter;
                std::string m_terminationInterval;
                std::string m_latestRetentionPeriod;
                std::string m_historyRetentionPeriod;

        };
    }
}
#endif
